      ////////////////////////////////////////
     // SPECSAISIE 1.2                      //
    // Written by Andrew Broad             ///
   //  BROADSOFT 2003                    ////
  //                                     /////
 // BinaryToTAP.java (2002:12:23 15:16) /////
////////////////////////////////////////////
// Methods of BinaryToTAP:              ///
// - static main                        //
/////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** BinaryToTAP is an application class for converting a raw binary file to a
** TAP file. The motivating example is to convert an SCR file to a TAP file
** containing a Spectrum SCREEN$ file.
*****************************************************************************/

public class BinaryToTAP {
	/********************************************************************/
	// static BinaryToTAP.main : String[] -> void
	/*********************************************************************
	** Application entry-point for converting a raw binary file to a TAP
	** file.
	** @param argv The vector of command-line arguments. There should be
	**             at least four of these: a binary file, the name of the
	**             TAP file to save, the name of the Spectrum file on the
	**             TAP file, the start-address, and an optional length
	**             which defaults to the length of the binary file.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length < 4 || argv.length > 5) {
			System.err.println("Usage: java BinaryToTAP "
			+ "<binary_file> <save_as.TAP> <Spectrum filename> "
			+ "<start_address> (<length>)");
			System.exit(1);
		}
		FileInputStream binary= null;
		try {
			binary= new FileInputStream(argv[0]);
		} catch (FileNotFoundException fnfe) {
			System.err.println("BinaryToTAP.main: cannot open \""
			+ argv[0] + "\"");
			System.exit(1);
		}
		int startAddress= 0;
		try {
			startAddress= Integer.parseInt(argv[3]);
		} catch (NumberFormatException nfe) {
			System.err.println(argv[3] + " doesn't parse as an "
			+ "integer!");
			System.exit(1);
		}
		int binaryLength= 0;
		try {
			binaryLength= binary.available();
		} catch (IOException ioe) {
			System.err.println("IOException thrown in " +
			                   "BinaryToTAP.main");
			System.exit(1);
		}
		int length= binaryLength;
		if (argv.length == 5) { 
			try {
				length= Integer.parseInt(argv[4]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[4] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		if (length > binaryLength) {
			length= binaryLength;
			System.out.println("Warning: length truncated to "
			                  + length);
		}
		Memory memory= new Memory();
		try {
			for (int i= 0; i < length; i++) {
				memory._memory[startAddress + i]
				= (byte)binary.read();
			}
		} catch (IOException ioe) {
			System.err.println("IOException thrown in " +
			                   "BinaryToTAP.main");
			System.exit(1);
		}

		Tape tape= new Tape();
		tape.addFile(memory.saveSpectrumCodeFile(argv[2],
		                            startAddress, length));
		tape.saveTAP(argv[1]);
		System.out.println("Saved " + argv[1]);
	}
}
