      ////////////////////////////////////
     // SPECSAISIE 1.2                  //
    // Written by Andrew Broad         ///
   //  BROADSOFT 2003                ////
  //                                 /////
 // CSaisie.java (2002:10:26 20:35) //////
//////////////////////////////////////////
// Methods of CSaisie:              /////
// - static csaisie                 ////
// - static processCluster          ///
// - static main                    //
/////////////////////////////////////

import java.io.*;

/*****************************************************************************
** CSaisie is an application class for generating Spectrum BASIC code for
** typing in code - useful for typing code in emulator format into a real
** Spectrum. This is the comparative version of Saisie, which takes an old
** file and a new file, and generates discriminative listings according to
** what has changed.
*****************************************************************************/

public class CSaisie {
	static boolean _includeREM= false;
	static String _filename;
	static boolean _outputToFiles= false;
	static PrintStream _out= System.out;
	static Memory _memory1;
	static Memory _memory2;

	/********************************************************************/
	// static CSaisie.csaisie : (int x int x int x int) -> void
	/*********************************************************************
	** Generates Spectrum BASIC code to type in a specified number of
	** bytes starting at the given start address.
	** @param start The address to start at.
	** @param length The number of bytes in the block.
	** @param distance The minimum distance between clusters.
	** @param density The minimum number of bytes in a cluster to trigger
	**                Saisie.
	*********************************************************************/

	private static void csaisie (int start, int length,
	                             int distance, int density)
	{
		int prev_a= start-1;
		int cluster_start= 0;
		int differences= 0;
		String cluster_output= "";
		for (int a= start; a <= start+length-1; a++) {
			if (_memory1._memory[a] != _memory2._memory[a]) {
			        if (cluster_start == 0)
			        	cluster_start= a;
				if ((prev_a != a - 1) && (prev_a != start-1))
					cluster_output+= "\n";
					// newline between non-contig. diffs.
				if (a - prev_a >= distance
				    && prev_a != start-1) {
					 if (processCluster(distance, density,
					                    differences,
					                    cluster_output,
					                    cluster_start,
					                    prev_a))
					{
						System.out.println();
					}
					cluster_output= "";
					differences= 0;
					cluster_start= a;
				}
			        differences++;
				cluster_output+= (a + ": " +
				 ByteSupport.unsignByte(_memory2._memory[a])
				 + "\n");
				prev_a= a;
			}
		}
		if (processCluster(distance, density, differences,
		                   cluster_output, cluster_start, prev_a)
		    && prev_a == start-1) {
			System.out.println();
		}
		if (prev_a != start-1) {
			System.out.println
			("---------------------------------------------------"
			+"---------------------------");
		}
	}

	/********************************************************************/
	// static CSaisie.processCluster : (int x int x String x int)
	//                                 -> boolean
	/*********************************************************************
	** Processes a cluster of differences that are at least
	** <TT>distance</TT> apart from the previous cluster, either
	** outputting the changes in Compare format, or calling saisie if
	** there are at least <TT>density</TT> changes in the cluster.
	** @param distance The minimum distance between clusters.
	** @param density The minimum number of bytes in a cluster to trigger
	**                Saisie.
	** @param differences The number of differences in the cluster.
	** @param clusterOutput The output (in Compare format) of the cluster.
	** @param clusterStart The address of the first difference in the
	**                     cluster.
	** @param clusterEnd The address of the last difference in the
	**                     cluster.
	** @return True if saisie was called, false if Compare output used.
	*********************************************************************/

	private static boolean processCluster (int distance, int density,
	                                       int differences,
	                                       String clusterOutput,
	                                       int clusterStart,
	                                       int clusterEnd) {
		if (differences >= density) {
			System.out.print(Saisie.saisie(_memory2,
			                 clusterStart, clusterEnd,
			                 true, _filename));
			return true;
		} else {
			System.out.print(clusterOutput);
			return false;
		}
	}

	/********************************************************************/
	// static CSaisie.main : String[] -> void
	/*********************************************************************
	** Application entry-point for Comparative Saisie. The program is
	** generated as text ready for printing out.
	** @param argv The vector of command-line arguments. There should be
	**             at least three arguments: two SNA files (old and new)
	**             and a start-address. The optional fourth argument is
	**             the length of code to print (i.e. the number of bytes
	**             to type in), which defaults to 1024 if not specified.
	**             The optional fifth argument is the stop-address, for
	**             when you want to generate more than one type-in at
	**             once. The optional sixth argument is the minimum
	**             distance between changed clusters, and the optional
	**             seventh argument is the minimum number of changes in
	**             each cluster to generate Saisie output rather than
	**             Compare output.
	*********************************************************************/

	public static void main (String argv[]) {
		String[] argv2= new String[argv.length];
		int argv2_length= 0;
		for (int i= 0; i < argv.length; i++) {
			argv2[argv2_length++]= argv[i];
		}

		if (argv2_length < 3 || argv2_length > 7) {
			System.err.println("Usage: java CSaisie <old_file.SNA"
			+ "> <new_file.SNA> <start_address> (<length> (<stop_"
			+ "address> (<distance> (<density>))))");
			System.exit(1);
		}
		_filename= argv2[1];

		int startAddress= 32768;
		try {
			startAddress= Integer.parseInt(argv2[2]);
		} catch (NumberFormatException nfe) {
			System.err.println(argv2[2] + " doesn't parse as an "
			+ "integer!");
			System.exit(1);
		}

		int length= 1024;
		if (argv2_length >= 4) {
			try {
				length= Integer.parseInt(argv2[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv2[3] + " doesn't parse"
				+ " as an integer!");
				System.exit(1);
			}
		}

		int stopAddress= startAddress;
		if (argv2_length >= 5) {
			try {
				stopAddress= Integer.parseInt(argv[4]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[4] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}

		int distance= 64;
		if (argv2_length >= 6) {
			try {
				distance= Integer.parseInt(argv[5]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[5] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}

		int density= 17;
		if (argv2_length >= 7) {
			try {
				density= Integer.parseInt(argv[6]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[6] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}

		_memory1= new Memory();
		_memory1.loadSNA(argv2[0]);

		_memory2= new Memory();
		_memory2.loadSNA(argv2[1]);

		for (int address= startAddress; address <= stopAddress;
		                                address+= length)
		{
			if (_outputToFiles) {
				try {
					_out= new PrintStream
					      (new FileOutputStream
			 		      (String.valueOf(address)));
				} catch (FileNotFoundException fnfe) {
					System.err.println(fnfe);
					System.exit(1);
				}
			}
			csaisie(address,length,distance,density);
		}
	}
}
