      ////////////////////////////////////
     // SPECSAISIE 1.2                  //
    // Written by Andrew Broad         ///
   //  BROADSOFT 2003                ////
  //                                 /////
 // Compare.java (2002:10:26 20:49) //////
/////////////////////////////////////////
// Methods of Compare:              ////
// - static compare                 ///
// - static main                    //
/////////////////////////////////////

import java.io.*;

/*****************************************************************************
** Compare is an application class for the bytewise comparison of two SNA 
** files.
*****************************************************************************/

public class Compare {
	static Memory _memory1= new Memory();
	static Memory _memory2= new Memory();

	/********************************************************************/
	// static Compare.compare : (int x int) -> void
	/*********************************************************************
	** Compares the two SNA files between the given start and stop
	** addresses (inclusively). Prints a list of addresses for which the
	** values are different, and their values for the <i>second</i> file.
	** @param startAddress The address to start comparing.
	** @param stopAddress The address to stop comparing.
	*********************************************************************/

	public static void compare (int startAddress, int stopAddress) {
		int prev_a= 0;
		for (int a= startAddress; a <= stopAddress; a++) {
			if ((a % 1024 == 0) && (prev_a != 0)) {
				System.out.println
				("-------------------------------------------"
				+"-----------------------------------");
				prev_a= 0;
			}
			if (_memory1._memory[a] != _memory2._memory[a]) {
				if ((prev_a != a - 1) && (prev_a != 0))
					System.out.println();
				System.out.println(a + ": " +
				 ByteSupport.unsignByte(_memory2._memory[a]));
				prev_a= a;
			}
		}
	}

	/********************************************************************/
	// static Compare.main : String[] -> void
	/*********************************************************************
	** Application entry-point for bytewise comparison of two SNA files.
	** The files are specified as command-line arguments, and optional
	** third and fourth command-line arguments are used to specify the
	** start and stop addresses (inclusive) for the comparison. The
	** defaults are the beginning (16357) and end (65535) of the file.
	** The output is a list of the addresses for which the two files have
	** different values, and their values for the <i>second</i> file.
	** @param argv The vector of command-line arguments. The first two
	**             arguments must specify the SNA files to be compared.
	**             The third and fourth arguments (optional) specify the
	**             start and stop addresses for the comparison.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length < 2 || argv.length > 4) {
			System.err.println
				("Usage: java Compare <file1.SNA> <file2.SNA>"
				+ " (<start_address> (<stop_address>))");
			System.exit(1);
		}
		int startAddress= 16357;
		if (argv.length >= 3) {
			try {
				startAddress= Integer.parseInt(argv[2]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[2] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		int stopAddress= 65535;
		if (argv.length >= 4) {
			try {
				stopAddress= Integer.parseInt(argv[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[3] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		_memory1.loadSNA(argv[0]);
		_memory2.loadSNA(argv[1]);
		compare(startAddress, stopAddress);
	}
}