      //////////////////////////////////
     // SPECSAISIE                    //
    // Written by Andrew Broad       ///
   //  BROADSOFT 2000              ////
  //                               /////
 // Count.java (2000:05:04 17:24) //////
///////////////////////////////////////
// Methods of Count:              ////
// - static count                 ///
// - static main                  //
///////////////////////////////////

import java.io.*;

/*****************************************************************************
** Count is an application class for calculating hash totals for a snapshot -
** the sum of byte values over a range of memory addresses. This is useful for
** quick comparison of a file on a real Spectrum and an emulator file, for
** example.
*****************************************************************************/

public class Count {
	static Memory _memory;

	/********************************************************************/
	// static Count.count : (int x int x int) -> void
	/*********************************************************************
	** Generates hash totals by summing the values of addresses from the
	** start address to the stop address, with a given step size.
	** @param start The address to start counting at.
	** @param stop The address to stop counting after (the entire block
	**             which contains this address will be included in the
	**             count).
	** @param step The step size (in bytes).
	*********************************************************************/

	private static void count (int start, int stop, int step) {
		for (int a= start; a <= stop; a+= step) {
			int count= 0;
			for (int i= 0; i < step; i++) {
				try {
					count+= ByteSupport.unsignByte
					        (_memory._memory[a + i]);
				}
				catch (ArrayIndexOutOfBoundsException aioobe){
					System.err.println("Count failed for "
					+ "block " + a + ", because it ran of"
					+ "f the end of memory ;-(");
					System.exit(1);
				}
			}
			System.out.println(a + ": " + count);
		}
	}

	/********************************************************************/
	// static Count.main : String[] -> void
	/*********************************************************************
	** Application entry-point for generating hash totals for a snapshot.
	** @param argv The vector of command-line arguments. There should be
	**             at least one argument: an SNA file. The second, third
	**             and fourth arguments are optional: the second argument
	**             is the start address (default 32768), the third
	**             argument is the step size (in bytes, default 1024) and
	**             the fourth argument is the stop address (default
	**             65535).
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length < 1 || argv.length > 4) {
			System.err.println("Usage: java Count <file.SNA> "
			+ "(<start_address> (<step_size> (<stop_address>)))");
			System.exit(1);
		}
		int startAddress= 32768;
		if (argv.length >= 2) {
			try {
				startAddress= Integer.parseInt(argv[1]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[1] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		int stepSize= 1024;
		if (argv.length >= 3) {
			try {
				stepSize= Integer.parseInt(argv[2]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[2] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		int stopAddress= 65535;
		if (argv.length >= 4) {
			try {
				stopAddress= Integer.parseInt(argv[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[1] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		_memory= new Memory();
		_memory.loadSNA(argv[0]);
		count(startAddress,stopAddress,stepSize);
	}
}