      /////////////////////////////////////////
     // SPECSAISIE 1.1                       //
    // Written by Andrew Broad              ///
   //  BROADSOFT 2002                     ////
  //                                      /////
 // PrintHeaders.java (2002:09:14 20:16) //////
///////////////////////////////////////////////
// Methods of PrintHeaders:              /////
// - static printHeaders                 ////
// - static printHeader                  ///
// - static main                         //
//////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** PrintHeaders is an application class for printing header information from
** TAP files.
*****************************************************************************/

public class PrintHeaders {
	/********************************************************************/
	// static PrintHeaders.printHeaders : Tape -> void
	/*********************************************************************
	** Print the header information for each Spectrum file on the tape.
	** @param argv The tape.
	*********************************************************************/

	public static void printHeaders (Tape tape) {
		Enumeration files= tape.getFiles().elements();
		while (files.hasMoreElements())
			printHeader((SpectrumFile)files.nextElement());
	}

	/********************************************************************/
	// static PrintHeaders.printHeader : SpectrumFile -> void
	/*********************************************************************
	** Print the header information for a Spectrum file.
	** @param argv A Spectrum file.
	*********************************************************************/

	public static void printHeader (SpectrumFile file) {
		System.out.println("Name: \"" + file.getName() + "\"");
		System.out.print("Type: ");
		switch (file.getType()) {
			case 0:
				System.out.println("BASIC");
				System.out.println("Line: "
				+ file.getParameter1());
				System.out.println("VarA: "
				+ file.getParameter2());
				break;
			case 1:
				System.out.println("NUMBER ARRAY");
				System.out.println("Par1: "
				+ file.getParameter1());
				System.out.println("Par2: "
				+ file.getParameter2());
				break;
			case 2:
				System.out.println("CHARACTER ARRAY");
				System.out.println("Par1: "
				+ file.getParameter1());
				System.out.println("Par2: "
				+ file.getParameter2());
				break;
			case 3:
				System.out.println("CODE");
				System.out.println("Star: "
				+ file.getParameter1() + " (Stop: "
				+ (file.getParameter1() + file.getLength()-1)
				+ ")");
				break;
		}
		System.out.println("Leng: " + file.getLength());
		System.out.println();
	}

	/********************************************************************/
	// static PrintHeaders.main : String[] -> void
	/*********************************************************************
	** Application entry-point for printing header information for each
	** Spectrum file in a TAP file.
	** @param argv The vector of command-line arguments. There should be
	**             one argument: the TAP file to print headers for.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length != 1) {
			System.err.println
			("Usage: java PrintHeaders <file.TAP>");
			System.exit(1);
		}
		System.out.println();
		printHeaders(new Tape(argv[0]));
	}
}