      ////////////////////////////////////////
     // SPECSAISIE 1.2                      //
    // Written by Andrew Broad             ///
   //  BROADSOFT 2003                    ////
  //                                     /////
 // SNAtoBinary.java (2002:12:23 15:42) /////
////////////////////////////////////////////
// Methods of SNAtoBinary:              ///
// - static main                        //
/////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** SNAtoBinary is an application class for loading SNA snapshot files into
** Spectrum memory and saving them as raw binary files.
*****************************************************************************/

public class SNAtoBinary {
	/********************************************************************/
	// static SNAtoBinary.main : String[] -> void
	/*********************************************************************
	** Application entry-point for converting SNA files to raw binary.
	** @param argv The vector of command-line arguments. There should be
	**             five arguments: the SNA file, the name of the TAP file
	**             to save, the Spectrum filename, the start address and
	**             the length (for a CODE file) or an optional auto-start
	**             line-number (for a BASIC file).
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length != 4) {
			System.err.print
			("Usage: java SNAtoBinary <sna> <binary filename> "
			+ "<start_address> <length>\n");
			System.exit(1);
		}
		int start= 0;
		try {
			start= Integer.parseInt(argv[2]);
		} catch (NumberFormatException nfe) {
			System.err.println(argv[2] + " doesn't parse as an "
			+ "integer!");
			System.exit(1);
		}
		int length= 0;
		try {
			length= Integer.parseInt(argv[3]);
		} catch (NumberFormatException nfe) {
			System.err.println(argv[3] + " doesn't parse "
			+ "as an integer!");
			System.exit(1);
		}
		Memory memory= new Memory();
		memory.loadSNA(argv[0]);

		FileOutputStream binary= null;
		try {
			binary= new FileOutputStream(argv[1]);
		} catch (FileNotFoundException fnfe) {
			System.err.println("SNAtoBinary.main: cannot create\""
			+ argv[1] + "\"");
			System.exit(1);
		}
		try {
			for (int i= 0; i < length; i++) {
				binary.write(memory._memory[start + i]);
			}
		} catch (IOException ioe) {
			System.err.println("IOException thrown in " +
			                   "SNAtoBinary.main");
			System.exit(1);
		}
		System.out.println("Saved " + argv[1]);
	}
}
