      /////////////////////////////////////
     // SPECSAISIE 1.2                   //
    // Written by Andrew Broad          ///
   //  BROADSOFT 2003                 ////
  //                                  /////
 // SNAtoTAP.java (2002:12:23 15:34) /////
/////////////////////////////////////////
// Methods of SNAtoTAP:              ///
// - static main                     //
//////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** SNAtoTAP is an application class for loading SNA snapshot files into
** Spectrum memory and saving them as TAP files.
*****************************************************************************/

public class SNAtoTAP {
	/********************************************************************/
	// static SNAtoTAP.main : String[] -> void
	/*********************************************************************
	** Application entry-point for converting SNA files to TAP files.
	** @param argv The vector of command-line arguments. There should be
	**             five arguments: the SNA file, the name of the TAP file
	**             to save, the Spectrum filename, the start address and
	**             the length (for a CODE file) or an optional auto-start
	**             line-number (for a BASIC file).
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length < 3 || argv.length > 5) {
			System.err.print
			("Usage: java SNAtoTAP <sna> <tap> <Spectrum filename"
			+ "> <start> <length>\n"
			+ "where         <sna>: SNA file\n"
			+ "              <tap>: TAP file to save\n"
			+ "<Spectrum filename>: name of CODE file\n"
			+ "            <start>: start-address\n"
			+ "           <length>: length (in bytes)\n\n"
			+ "   OR: java SNAtoTAP <sna> <tap> <Spectrum filenam"
			+ "e> (<line>)\n"
			+ "where         <sna>: SNA file\n"
			+ "              <tap>: TAP file to save\n"
			+ "<Spectrum filename>: name of BASIC file\n"
			+ "             <line>: auto-start line-number\n");
			System.exit(1);
		}
		int line= 32768;
		if (argv.length == 4) {
			try {
				line= Integer.parseInt(argv[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[3] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		int start= 0;
		if (argv.length == 5) {
			try {
				start= Integer.parseInt(argv[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[3] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		int length= 0;
		if (argv.length == 5) {
			try {
				length= Integer.parseInt(argv[4]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[4] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}
		Memory memory= new Memory();
		memory.loadSNA(argv[0]);

		Tape tape= new Tape();
		if (argv.length == 5) {
			tape.addFile(memory.saveSpectrumCodeFile
			                    (argv[2], start, length));
		} else {
			tape.addFile(memory.saveSpectrumBasicFile
			                    (argv[2], line));
		}
		tape.saveTAP(argv[1]);
		System.out.println("Saved " + argv[1]);
	}
}