      //////////////////////////////////
     // SPECSAISIE                    //
    // Written by Andrew Broad       ///
   //  BROADSOFT 2000              ////
  //                               /////
 // Split.java (2000:05:04 17:26) //////
///////////////////////////////////////
// Methods of Split:              ////
// - static deconcatenate         ///
// - static main                  //
///////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** Split is an application class for the deconcatenation of a TAP file into a 
** separate TAP file for each Spectrum file on the tape.
*****************************************************************************/

public class Split {
	/********************************************************************/
	// static Split.deconcatenate : Tape -> void
	/*********************************************************************
	** Saves each Spectrum file on a tape as a separate TAP file.
	** @param tape The tape to deconcatenate.
	*********************************************************************/

	private static void deconcatenate (Tape tape) {
		String nameRoot= tape.getName().
		                 substring(0, tape.getName().length() - 4);

		Enumeration files= tape.getFiles().elements();
		short counter= 0;
		while (files.hasMoreElements()) {
			++counter;
			Tape subtape= new Tape();
			subtape.addFile((SpectrumFile)files.nextElement());
			String filename= nameRoot + "_" + counter + ".TAP";
			subtape.saveTAP(filename);
			System.out.println("Saved " + filename);
		}
	}

	/********************************************************************/
	// static Split.main : String[] -> void
	/*********************************************************************
	** Application entry-point for deconcatenation of a TAP file into a
	** separate TAP file for each Spectrum file on the tape. For example,
	** consider a TAP file MM.TAP containing three Spectrum files.
	** "java Split MM.TAP" would save three new TAP files: MM_1.TAP,
	** MM_2.TAP and MM_3.TAP, such that "cat MM_1.TAP MM_2.TAP MM_3.TAP"
	** produces a file which is identical to MM.TAP.
	** @param argv The vector of command-line arguments. There should be
	**             one argument: the TAP file to deconcatenate.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length != 1) {
			System.err.println("Usage: java Split <file.TAP>");
			System.exit(1);
		}
		deconcatenate(new Tape(argv[0]));
	}
}