      ////////////////////////////////////////
     // SPECSAISIE                          //
    // Written by Andrew Broad             ///
   //  BROADSOFT 2000                    ////
  //                                     /////
 // TAPtoBinary.java (2000:05:04 17:26) /////
////////////////////////////////////////////
// Methods of TAPtoBinary:              ///
// - static main                        //
/////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** TAPtoBinary is an application class for converting a TAP file to a raw
** binary file. The motivating example is to convert a Spectrum SCREEN$ file
** on a TAP file to a SCR file.
*****************************************************************************/

public class TAPtoBinary {
	/********************************************************************/
	// static TAPtoBinary.main : String[] -> void
	/*********************************************************************
	** Application entry-point for converting a TAP file to a raw binary
	** file. If there is more than one Spectrum file on the TAP file, this
	** program only looks at the first Spectrum file (to convert all the
	** Spectrum files to raw binaries, use Split first).
	** @param argv The vector of command-line arguments. There should be
        **             two of these: a TAP file containing a Spectrum file,
        **             and the name of the binary file to save.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length != 2) {
			System.err.println("Usage: java TAPtoBinary "
			+ "<file.TAP> <binary_file_name>");
			System.exit(1);
		}
		Tape tape= new Tape(argv[0]);
		SpectrumFile file= (SpectrumFile)tape.getFiles().elementAt(0);
		FileOutputStream binary= null;
		try {
			binary= new FileOutputStream(argv[1]);
		} catch (FileNotFoundException fnfe) {
			System.err.println("TAPtoBinary.main: cannot create\""
			+ argv[1] + "\"");
			System.exit(1);
		}
		try {
			for (int i= 0; i < file.getLength(); i++) {
				binary.write(file._dataBlock[3 + i]);
			}
		} catch (IOException ioe) {
			System.err.println("IOException thrown in " +
			                   "TAPtoBinary.main");
			System.exit(1);
		}
		System.out.println("Saved " + argv[1]);
	}
}