      /////////////////////////////////////
     // SPECSAISIE 1.2                   //
    // Written by Andrew Broad          ///
   //  BROADSOFT 2003                 ////
  //                                  /////
 // TAPtoSNA.java (2002:12:23 15:41) /////
/////////////////////////////////////////
// Methods of TAPtoSNA:              ///
// - static main                     //
//////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** TAPtoSNA is an application class for loading TAP files into Spectrum
** memory and saving them as SNA snapshot files.
*****************************************************************************/

public class TAPtoSNA {
	/********************************************************************/
	// static TAPtoSNA.main : String[] -> void
	/*********************************************************************
	** Application entry-point for converting TAP files to SNA files.
	** Currently a crude approximation, which only works for CODE files
	** and does not set any system variables.
	** @param argv The vector of command-line arguments. There should be
	**             three arguments: an SNA file encoding the initial state
	**             of memory, a TAP file, and the SNA file to save the
	**             result of loading the TAP file into memory as.
	*********************************************************************/

	public static void main (String argv[]) {
		if (argv.length < 3 || argv.length > 4) {
			System.err.print
			("Usage: java TAPtoSNA <sna0> <tap> <sna1> (<addr>)\n"
			+ " where <sna0>: SNA file representing initial state"
			+ "\n        <tap>: TAP file to load\n"
			+ "       <sna1>: SNA file representing state after "
			+ "loading TAP file\n"
			+ "       <addr>: Optional address to relocate code "
			+ "files to\n");
			System.exit(1);
		}
		boolean relocate= false; int address= 0;
		if (argv.length == 4) {
			try {
			        relocate= true;
				address= Integer.parseInt(argv[3]);
			} catch (NumberFormatException nfe) {
				System.err.println(argv[3] + " doesn't parse "
				+ "as an integer!");
				System.exit(1);
			}
		}

		Memory memory= new Memory();
		memory.loadSNA(argv[0]);

		Tape tape= new Tape(argv[1]);
		ListIterator files= tape.getFiles().listIterator();
		while (files.hasNext()) {
			SpectrumFile file= (SpectrumFile)files.next();
			switch (file.getType()) {
			case SpectrumFile.BASIC:
				int PROG= memory.deek(23635);
				memory.doke(23627, PROG+file.getParameter2());
				memory.loadSpectrumFile(file, PROG);
				break;
			case SpectrumFile.CODE:
				if (relocate)
					memory.loadSpectrumFile(file,address);
				else
					memory.loadSpectrumFile(file);
				break;
			default:
				System.err.println("Warning: TAPtoSNA does "
				+ "not recognise the file-type of \""
				+ file.getName() + "\" - only BASIC and CODE "
				+ "files are currently supported. That file "
				+ "has been ignored.");
			}
		}
		memory.saveSNA(argv[2]);
	}
}
